/*
	ef_Duplicate
	
	Copies, pastes and selects pasted geometry.
	Requested by NT forum member Danner. Thanks!
	
	Use at own risk.
	
	author Dan Dulberger evenflcw@yahoo.com
	
	version log
	v1.0 (2011-03-19)	-Initial release.
	v1.1 (2011-03-19)	-Fixed bug related to selmode(direct)
						claiming polys selected in POINT_MODE mode.
						-Added som info calls so it's abit easier
						to note whan operation is finished.
						-Fixed mistake in array initialization.
	v1.2 (2011-12-05)	-Fixed handling when multiple layers are
						in the foreground.
	v1.3 (2011-12-06)	-Fixed bug. Previously used Mesh() (first object) rather than Mesh(0) (active object)! Doh!
						-Fixed bug. Worked around Lscript bug in Mesh OA pointCount and polygonCount methods. Albeit it won't manifest itself in this script.
*/
@script modeler
@version 2.7
@name Duplicate



@define	POINT_MODE 	 1 
@define POLYGON_MODE 2



main {

	mesh = Mesh(0);
	layers = lyrfg(); 

	undogroupbegin();
	{
		selpolcnt = selpolycount();
		selpntcnt = selpointcount();
		if(!selpolcnt && !selpntcnt)
		{
			info("No points or polygons selected.");
		}	
		else
		{
			globpntcnt = layerpointcount(mesh, layers[1]); //LW pastes into primary layer, so that's what we have to keep count on.
			globpolcnt = layerpolycount(mesh, layers[1]);
			
			selmode(USER);
			copy();
			paste();
			
			mode = selpolcnt ? POLYGON_MODE : POINT_MODE;			
			switch(mode)
			{
				case POLYGON_MODE:
					selpolygon(CLEAR);
					select_polygons(globpolcnt+1, globpolcnt+selpolcnt);
					info(selpolcnt, " polygons duplicated.");
					break;
					
				case POINT_MODE:
					selpoint(CLEAR);
					select_points(globpntcnt+1, globpntcnt+selpntcnt);
					info(selpntcnt, " points duplicated.");
					break;
			}
		}
	}
	undogroupend();
}

set_selection_mode: element_type {
	switch(element_type)
	{
		case POINT_MODE:
			selpoint(SET, NPEQ, -1);
			break;
		
		case POLYGON_MODE:
			selpolygon(SET, NVEQ, 0);
			break;
	}
}

select_polygons: ndx_start, ndx_end
{
	var elements[ndx_end-ndx_start+1];
	for(i=1, n=ndx_start; n<=ndx_end; i++, n++)
	{
		elements[i] = n;
	}
	selpolygon(SET, POLYNDX, elements);
}

select_points: ndx_start, ndx_end
{
	var elements[ndx_end-ndx_start+1];
	for(i=1, n=ndx_start; n<=ndx_end; i++, n++)
	{
		elements[i] = n;
	}
	selpoint(SET, POINTNDX, elements);
}

globalpointcount {
	selmode(GLOBAL);
	gcount = pointcount();
	return gcount;
}

globalpolycount {
	selmode(GLOBAL);
	(gcount,,,,,) = polycount();
	return gcount;
}

layerpointcount: mesh, layer_index {
	count = mesh.pointCount(layer_index);
	if(count==1) { //bug handling, pointCount/polygonCount method will return 1 for empty layer.
		empty_layers = lyremptyfg();
		if(empty_layers.contains(layer_index)) {
			count = 0;
		}
	}
	return count;
}

layerpolycount: mesh, layer_index {
	count = mesh.polygonCount(layer_index);
	if(count==1) { //bug handling, pointCount/polygonCount method will return 1 for empty layer.
		empty_layers = lyremptyfg();
		if(empty_layers.contains(layer_index)) {
			count = 0;
		}
	}
	return count;
}

selpointcount {
	selmode(USER);
	ucount = pointcount();
	selmode(DIRECT);
	dcount = pointcount();
	
	if(ucount==dcount) return ucount;
	else return 0;
}

selpolycount {
//handles situation were selmode:direct will indicate polygons selected in pointmode
	selmode(USER);
	(ucount,,,,,) = polycount();
	selmode(DIRECT);
	(dcount,,,,,) = polycount();
	
	if(ucount==dcount) return ucount;
	else return 0;
}
